<?php
/*
Plugin Name: Simple Input Tabs
Description: Adds additional user inputs where required in the theme.
Version: 1.5b
Author: Jason Dixon
Author URI: http://simpleextrainputs.internetimagery.com
*/

/*	---------------------------------------------------------------------------------
*	IF YOU WISH TO CHANGE THE COLOUR OF THE CURRENTLY ACTIVE TAB, EDIT THIS VARIABLE.
*	---------------------------------------------------------------------------------*/
	$SIT_CURRENTLY_ACTIVE_TAB = '#96f4b4';
/*	---------------------------------------------------------------------------------*/

/*	---------------------------------------------------------------------------------
*	IF YOU WANT TO GIVE PERMISSION TO HAVE PLUGINS PLACE DATA ON TOP OR BELOW YOUR
*	!!EXTRA!! CONTENT, MAKE THESE VALUES TRUE. (PLUGINS CAN STILL PLACE DATA ABOVE OR
*	BELOW YOUR 'MAIN' CONTENT SECTION REGARDLESS OF THIS SETTING.
*	---------------------------------------------------------------------------------*/
	$SIT_CONTENT_TOP	= false;
	$SIT_CONTENT_BOTTOM	= false;
/*	---------------------------------------------------------------------------------*/


# get correct id for plugin
$thisfile=basename(__FILE__, '.php');

# register plugin
register_plugin(
	$thisfile, //Plugin id
	'Simple Input Tabs', 	//Plugin name
	'1.5b', 		//Plugin version
	'Jason Dixon',  //Plugin author
	'http://simpleextrainputs.internetimagery.com', //author website
	'Adds additional user inputs where required in the theme', //Plugin description
	'', //page type - on which admin tab to display
	''  //main function (administration)
);
// Sort out hooks.
add_action('edit-extras','page_edit_box');//					Process content and choose which data to display.
add_action('edit-content','page_edit_foot');//					Add some new page tabs below the content edit box.
add_action('changedata-save','intercept_save_file');//				Intercept file save before it is saved.
add_filter('content','content_hostage');//					Intercept content before it is put to page.
add_filter('theme-header','extract_raw_content_data');//			Parse the data before any requests are made.


/*	---------------------------------------------------------------------------------
*	Functions below are reserved for use in template files by template creators.
*	The idea is to have the option to keep consistency by using a naming convention.
*	You can still use GetSimpleCMS's normal functions, and they will do what they have always done.
*	get_page_content($slug) functions differently to insert_page_content($field) however.
*	---------------------------------------------------------------------------------*/

function insert_page_content($field = NULL,$blurb=NULL){//			Output users content to page
	fill_in_the_tabs($field,$blurb);}//					Use -> to output a different pages content.

function insert_page_header(){//						Output header information
	get_page_header();}

function insert_page_footer(){//						Output footer information
	get_page_footer();}

function insert_page_navigation($nav=NULL){//					Output page navigation
	get_navigation($nav);}

function return_tab_content($location){//					Return a specific tabs content for processing.
	$query = explode('->', $location);
	if (count($query)==2){
	return hunt_page_data($query[0], $query[1]);}
	else { return speak_to_me('TAB_RETURN_PROBLEM');}
}	/*-------------------------------------------------------------------------------*/


/*	---------------------------------------------------------------------------------
*	Check if there is an external language file, and if so use it.
*	Otherwise use one of the built in languages.
*	---------------------------------------------------------------------------------*/
if(file_exists(GSPLUGINPATH . 'simple_input_tabs/lang/'.$LANG.'.php')){
	$SIT_LANG_EXTERNAL = true;//						Check if there is an external language file,
	i18n_merge('simple_input_tabs');}//					if not then use a local language.
else{
	$SIT_LANG_EXTERNAL = false;
	i18n_internal_language();}

function speak_to_me($words){//							Translate the request into the language.
	global $SIT_LANG_EXTERNAL;//						Get language location.
	if($SIT_LANG_EXTERNAL == true){//					Do we have an external language file?
		return i18n_r('simple_input_tabs/'.$words);}
		else{//								Guess it must be a local language.
		global $SIT_I18N_LOCAL;
		return $SIT_I18N_LOCAL[$words];}
}	/*-------------------------------------------------------------------------------*/


/*	---------------------------------------------------------------------------------
*	Triggers that happen on the page edit screen.
*	Saving page information to file and recording the tab last used etc.
*	---------------------------------------------------------------------------------*/
function page_edit_box(){//							Process data and display in the editor.

	global $SIT_LAST_TAB;//							Set a page to come back to after saving.
	global $SIT_TAB_LIST;//							Initialize our tab list.
	global $SIT_WARNING;//							Warning message.
	global $data_edit;//							Grab the XML page file.
	global $TEMPLATE;//							Get theme folder.
	global $template;//							Get theme file.
	global $content;//							Get page content.
	$SIT_WARNING = 0;//							Preemptively set it to false. Warnings suck!

	$SIT_TAB_LIST = scan_template($TEMPLATE,$template,'insert_page_content');//Scan the template folder, gather content requests.
	if(isset($data_edit->returnpage)){//					Check to see if a return page exists.
	$SIT_LAST_TAB = (string)$data_edit->returnpage;//			If not, then set it to the default.
	} else {$SIT_LAST_TAB = speak_to_me('DEFAULT_TAB');}
	if(!array_key_exists($SIT_LAST_TAB,$SIT_TAB_LIST)){//			Check to see if the return tab is in the template.
		$SIT_LAST_TAB = key($SIT_TAB_LIST);}//				If not, then swap to a relevant tab.

	echo '<!--- '.speak_to_me('SIT_ADDED')." ---> \n";//	Store record of the last tab viewed.
	echo '<input type="hidden" name="lastpage-view" value="'.safe_slash_html($SIT_LAST_TAB).'">';
	//									Store raw content data for retrieval later.
	echo '<input type="hidden" name="old-page-content" value="'.safe_slash_html($content).'">';

	$display_content = scan_content_string($content);//			Extract the data from the page xml file.
	if($content == NULL){//			If the page is fresh add in welcome message.
		$content = '<p></p><h1>'.speak_to_me('WELCOME_NEW_PAGE').'</h1><p>'.speak_to_me('WELCOME_PAGE_BLURB').'</p>';}
		else {//					Leave the content alone. Otherwise if there is data,
	$content = $display_content[$SIT_LAST_TAB];}//				pick the correct tab data to display to the user.
	if(trim($content) == NULL){//						If the content is a new tab, throw in a message.
		if($SIT_TAB_LIST == NULL){$content = $display_content[speak_to_me('DEFAULT_TAB')];}
		else if($SIT_TAB_LIST[$SIT_LAST_TAB] == NULL){
		$content = speak_to_me('WELCOME_TAB_BLURB').' '.$SIT_LAST_TAB;}
		else { $content = $SIT_TAB_LIST[$SIT_LAST_TAB];}}
}
function page_edit_foot(){//							Print out tabs below the text input box.

	global $SIT_CURRENTLY_ACTIVE_TAB;//					Get the active tab style.
	global $SIT_LAST_TAB;//							Get the active tab name.
	global $SIT_TAB_LIST;//							Get the list of tab names.
	global $SIT_WARNING;//							Check the warning.
	global $parent;//							Access the pages URL for display.
	global $url;//								Access the pages parents for display.

	if($SIT_WARNING != 0){echo '<div class="updated">'.speak_to_me('WARNING').' ';//Is there a warning?
	if($SIT_WARNING == 1){//						There is a duplicate tab in the theme.
	echo speak_to_me('DUPLICATE_TABS');}
	if($SIT_WARNING == 2){//						There are illegal tabs hidden.
	echo speak_to_me('ILLEGAL_TABS');}
	echo '</div>';}
	echo '<!--- '.speak_to_me('SIT_ADDED')." ---> \n <p>";//		Create some tabs for each input.
	foreach($SIT_TAB_LIST as $tab_name=>$value){
	$active_tab = '';
	$shadow_details = 'inset 0px 2px 3px 0px #acacac';//			Make a fancy shadow for the selected tab.
	if($tab_name == $SIT_LAST_TAB){//					Make a special style for the active tab,
		$active_tab = $SIT_CURRENTLY_ACTIVE_TAB;//			and make the tab un-clickable.
		echo '<span><input class="submit" style="background:'.$active_tab.'; -webkit-box-shadow: '.$shadow_details.'; -moz-box-shadow:'.$shadow_details.'; box-shadow: '.$shadow_details.';" type="button" name="submitted" value="'.$tab_name.'" onclick="warnme=false;" /></span>';
	} else {
	echo '<span><input class="submit" type="submit" name="submitted" value="'.$tab_name.'" onclick="warnme=false;" /></span>';}
	}
	echo '</p>';
	if(count($SIT_TAB_LIST)>1){//						Only display this info if there is more than one tab.
	echo speak_to_me('CURRENTLY_EDITING').' <strong><span style="color:green">'.$SIT_LAST_TAB.'</span></strong>.<br/>';}
	echo speak_to_me('CHANGE_TEMPLATE').'<br></p>';
	//									Add a separate button that dispalys tab locations.
	echo '<p><img src="template/images/search.png"/> <a href="'.trim(find_url($url, $parent)).'?display=preview" target="_blank" >'.speak_to_me('PREVIEW_BUTTON').'</a></p>';
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Intercepting the saving of the page file.
*	Extracting the users input data, and adding our own tags to it.
*	Sending back the result!
*	---------------------------------------------------------------------------------*/
function intercept_save_file(){
	global $metad;//							Nick off with meta data!
	global $xml;//								Hijack the page save file.
	if(isset($_POST['submitted'])){//					Probably pointless to check, but best to be safe.
		if($_POST['lastpage-view'] != NULL){
		$last_tab = $_POST['lastpage-view'];}//				Get the last tab that was open.
		else{$last_tab = speak_to_me('DEFAULT_TAB');}
		if($_POST['submitted'] == i18n_r('BTN_SAVEUPDATES') || $_POST['submitted'] == i18n_r('BTN_SAVEPAGE')){
			$return_tab = $last_tab;//				Check to see if the page has just been saved
		} else {//							or if the tab has been changed.
			$return_tab = $_POST['submitted'];}//			Get the last tab viewed, so we can return there.
		$last = $xml->addChild('returnpage');//				Insert record of last tab viewed.
		$last->addCData($return_tab);

		$new_information = (string)$xml->content[0];//			Pull out the newly entered information.
		$old_information = scan_content_string($_POST['old-page-content']);//Original page information.
		$old_information[$last_tab] = $new_information;//		insert the new information replacing the old.
		$save_info = $old_information[speak_to_me('DEFAULT_TAB')];
		foreach($old_information as $k=>$v){//				Compile data back into single string.
			if($k != speak_to_me('DEFAULT_TAB')){
			$each_tab = safe_slash_html('||]'.$k.'[||');//		Ensure the tab name is secure with safe_slash_html.
			$save_info .= $each_tab.$v;}//				Put em together.
		}
		$xml->content[0] = '';//					Clear the XML data.
		$xml->content[0]->addCData($save_info);//			Insert our altered information.

	if(trim($metad) == NULL){//						Was the page description left empty? Fill it in!
		if(array_key_exists(speak_to_me('DEFAULT_TAB'), $old_information)){
			$new_description = $old_information[speak_to_me('DEFAULT_TAB')];}
		else { $new_description = reset($old_information);}
		$new_description = trim(substr(strip_tags(strip_decode(html_entity_decode($new_description))), 0, 160));
		$new_description = str_replace(array("\r\n", "\r", "\n", "\t"), '', $new_description);
		$xml->metad[0] = '';
		$xml->metad[0]->addCData($new_description);}//			Add in description.
	}
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Find and return the contents of a specified page / tab combination.
*	---------------------------------------------------------------------------------*/
function hunt_page_data($page=NULL,$tab=NULL){
	extract_raw_content_data();
	if (trim($tab == NULL)){$tab = speak_to_me('DEFAULT_TAB');}//		Set default file.
	if (trim($page==NULL)){
		global $SIT_RAW_CONTENT_DATA;
		$unpacked = $SIT_RAW_CONTENT_DATA;}
	else {
	$file = getXML(GSDATAPAGESPATH . $page . '.xml');//			Find the page file and load it.
	$content = strip_decode($file->content[0]);//				Get the content from the file.
	$unpacked = scan_content_string($content);}//				Extract the data from content.
	$tab = bling($tab);
	if(array_key_exists($tab, $unpacked)){return $unpacked[$tab];}
	else if (count($unpacked)==1){return end($unpacked);}
	else { return speak_to_me('ERROR404');}//Send back the data.
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Get a list of all pages.
*	---------------------------------------------------------------------------------*/
function list_all_pages(){
	$large_list = get_available_pages();
	for($i=0;$i<=(count($large_list)-1);$i++){
		$shortlist[$i] = bling($large_list[$i]['slug']);}
		return $shortlist;
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Parse the content data. Return values from ||] ... [|| Returns false if the string
*	does not yet have anything to parse.
*	---------------------------------------------------------------------------------*/
function scan_content_string($string){
$string_array = explode('||', $string);//					Break up our string.
$content = NULL;//								Set an empty data variable. Lets fill it!
$previous_tab = NULL;//								Set tab memory.
$array_location = 'Beginning';//						Set string location.
foreach($string_array as $item){//						Begin extracting the content data.
	if (substr($item,0,1) == ']' && substr($item,-1) == '['){ //		Is the current item a tab?
		$item = str_replace(']', '', $item);
		$item = str_replace('[', '', $item);//				Take out the ] [ characters.
		$content[$item] = '';
		$previous_tab = $item;//					Save this tab in the memory, for next string.
		$array_location = 'Tab';}//					Where are we in the data? We are working with tabs.
	else {//								Guess the item is not a tab...
		if ($array_location == 'Tab'){//				Last item was a tab. This must then be the data.
			$content[$previous_tab]=$item;//			Add content to the key of the previous tab.
			$array_location = 'Data';}//				We are now in the data.
		else if($array_location == 'Data'){//				We are still in data. We accidentally grabbed ||.
			$content[$previous_tab].= '||'.$item;}//			Put them back in. Clean up our messes.
		else if($array_location == 'Beginning'){//			If we are at the beginning, make it the default tab.
			$content[speak_to_me('DEFAULT_TAB')] = $item;
			$previous_tab = speak_to_me('DEFAULT_TAB');
			$array_location = 'Data';}
		}}
	return $content;//						Return the data!
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Parse the template file. Return values from within insert_page_content( ... );
*	Return with default tab if there are no calls to this plugin on the template.
*	---------------------------------------------------------------------------------*/
function scan_template($theme_folder,$theme_file,$function){
	global $SIT_WARNING;//							Establish warnings.
	$validated_list = array();//						Establish empty array.
	$template_url = GSTHEMESPATH . $theme_folder . '/' . $theme_file;//	Find the template file.
	$the_file = file_get_contents($template_url);//				And scan the file.
	preg_match_all("/<\?php.*?\b".$function."\(([^)]*?)\);?.*?\?>/si", $the_file, $result_list);
	$search = array('"',"'"); //						Things we want to remove. Brackets.
	foreach($result_list[1] as $value){//			Go through and make a new list.
		$value = explode(',', str_replace($search, '', $value),2);
		$tab_value = explode('->', $value[0]);
		$tab = bling($tab_value[0]);//					Pull out the tab name.
		if($tab == NULL){$tab = speak_to_me('DEFAULT_TAB');}//

		if(count($tab_value)>1){}//					If the tab has "->" then dump it.
		else if(!preg_match('/^[\w_-\s]+$/',$tab)){//		Check the tab if it has illegal characters.
		$SIT_WARNING = 2;}//						Warn that the tab is illegal and dump it.
		else if(array_key_exists($tab, $validated_list)){//		Check for duplicates.
			$SIT_WARNING = 1;}//					Trigger duplicate warning.
		else { $validated_list[$tab] = $value[1];}//			Add tab to the list.
	}
	return $validated_list;
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Make text look pretty. Gotta look presentable!
*	---------------------------------------------------------------------------------*/
function bling($text){
	return trim(ucwords(strtolower($text)));
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Generate raw data from page content.
*	---------------------------------------------------------------------------------*/
function extract_raw_content_data(){
	global $content;
	if(!array_key_exists('SIT_RAW_CONTENT_DATA', $GLOBALS)){
		$new_data = strip_decode($content);
		global $SIT_RAW_CONTENT_DATA;
		$SIT_RAW_CONTENT_DATA = scan_content_string($new_data);}
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Output the data to the page in the correct place!
*	---------------------------------------------------------------------------------*/
function fill_in_the_tabs($tab,$blurb){
	global $SIT_CONTENT_BOTTOM;
	global $SIT_CONTENT_TOP;
	extract_raw_content_data();
	global $SIT_RAW_CONTENT_DATA;
	if (trim($tab) == NULL){$tab = speak_to_me('DEFAULT_TAB');}
	$field = array_map('trim', explode('->',$tab));
	if($_GET['display'] == 'preview'){
		echo '<center><h1>'.bling($tab).'</h1><p>'.$blurb.'</p></center>';}
	else if(count($field) == 2){
		echo hunt_page_data($field[0], $field[1]);}
	else if ($field[0] == speak_to_me('DEFAULT_TAB')){
		get_page_content();}
	else if (array_key_exists(bling($field[0]), $SIT_RAW_CONTENT_DATA)){
		if($SIT_CONTENT_TOP == true){exec_action('content-top');}
		echo exec_filter('content', $SIT_RAW_CONTENT_DATA[bling($field[0])]);
		if($SIT_CONTENT_BOTTOM == true){exec_action('content-bottom');}}
	else { echo speak_to_me('ERROR404');}
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	If someone calls get_page_content(), process content so they don't get garbage.
*	---------------------------------------------------------------------------------*/
function content_hostage($request){
	extract_raw_content_data();
	$unpacked = scan_content_string($request);//				Pull out the data from the content.
	if($_GET['display'] == 'preview'){// 					Are we looking at a preview?
		echo strip_decode('<center><h4>'.speak_to_me('SECTION_CODE').'<br>&amp;lt;?php get_page_content(); ?&amp;gt;<br>'.speak_to_me('REPLACE_CODE').'<br>&amp;lt;?php insert_page_content(); ?&amp;gt;</h4></center>');}
	else if(array_key_exists(speak_to_me('DEFAULT_TAB'), $unpacked)){//	Check if the content has the default tab "Main"
			return $unpacked[speak_to_me('DEFAULT_TAB')];}//	If so, send that out to them.
		else { return speak_to_me('ERROR404');}//			If there is no content then send none.
}	/*-------------------------------------------------------------------------------*/


/*	----------------------------------------------------------------------------------
*	Built in 118n Language text. Can be translated directly in here if desired.
*	Simply copy/paste the code between "case" and "break" and add in your own translations.
*	---------------------------------------------------------------------------------*/
function i18n_internal_language(){
global $SIT_I18N_LOCAL;
global $LANG;
switch ($LANG)
{
// TRANSLATE ME!!
case '': // insert language "name" ie: en_US for english.
$SIT_I18N_LOCAL = array(
// Nice little message in the source code
'SIT_ADDED' => '',
//name of default tab!
'DEFAULT_TAB' => '',
// Welcome messages that appear when new pages and/or tabs are opened.
'WELCOME_NEW_PAGE' => '',
'WELCOME_PAGE_BLURB' => '',
'WELCOME_TAB_BLURB' => '',
//error messages
'ERROR404' => '',
'TAB_RETURN_PROBLEM' => '',
'DUPLICATE_TABS' => '',
'ILLEGAL_TABS' => '',
'WARNING' => '',
//helpful messages
'CURRENTLY_EDITING' => '',
'CHANGE_TEMPLATE' => '',
'TAB_SECTION_NAME' => '',
'PREVIEW_BUTTON' => '',
// preview page
'SECTION_CODE' => '',
'REPLACE_CODE' => ''
);
break;

// Spanish
// Translated by islander. Thank you.

case 'es_ES': // insert language "name" ie: en_US for english.
$SIT_I18N_LOCAL = array(
// Nice little message in the source code
'SIT_ADDED' => 'Agregado por el plugin de Simple Input Tabs',
//name of default tab!
'DEFAULT_TAB' => 'Principal',
// Welcome messages that appear when new pages and/or tabs are opened.
'WELCOME_NEW_PAGE' => '&iexcl;Bienvenidos a la nueva p&aacute;gina!',
'WELCOME_PAGE_BLURB' => 'Quitar este texto y a&ntilde;adir su contenido aqu&iacute;. Abajo hay unas pesta&ntilde;as nombradas. &Uacute;salos para agregar contenido a las diferentes &aacute;reas de su p&aacute;gina web. No se olvide de a&ntilde;adir un t&iacute;tulo a su p&aacute;gina, arriba.',
'WELCOME_TAB_BLURB' => 'A&ntilde;adir el contenido de la secci&oacute;n:',
//error messages
'ERROR404' => 'Error 404: El contenido solicitado no fue encontrado.',
'TAB_RETURN_PROBLEM' => 'Hubo un problema con su pedido de pesta&ntilde;a.',
'DUPLICATE_TABS' => 'Su plantilla seleccionada tiene pesta&ntilde;as duplicadas del mismo nombre.',
'ILLEGAL_TABS' => 'Hay una o m&aacute;s fichas de su plantilla que utilizan caracteres no v&aacute;lidos. Ellos han estado oculta.',
'WARNING' => '&iexcl;ADVERTENCIA!',
//helpful messages
'CURRENTLY_EDITING' => 'Al hacer clic en una pesta&ntilde;a de arriba guardar y cambiar las entradas. Est&aacute; editando el archivo:',
'CHANGE_TEMPLATE' => 'Si cambia la plantilla de p&aacute;gina, necesitar&aacute; que guardar para actualizar sus fichas.',
'TAB_SECTION_NAME' => 'El contenido de esta pesta&ntilde;a va en:',
'PREVIEW_BUTTON' => 'Mostrar ubicaciones de pesta&ntilde;as.',
// preview page
'SECTION_CODE' => 'Esta secci&oacute;n est&aacute; poblado por el c&oacute;digo:',
'REPLACE_CODE' => 'Para funcionar bien, debe cambiar el c&oacute;digo a:'
);
break;

// ENGLISH
default:
$SIT_I18N_LOCAL = array(
// Nice little message in the source code
'SIT_ADDED' => 'Added by the Simple Input Tabs Plugin',
//name of default tab!
'DEFAULT_TAB' => 'Main',
// Welcome messages that appear when new pages and/or tabs are opened.
'WELCOME_NEW_PAGE' => 'Welcome to your new page!',
'WELCOME_PAGE_BLURB' => 'Remove this text and add your content here. Below are some named Tabs. Use them to add content to different areas of your web page. Don\'t forget to add a title to your page, above.',
'WELCOME_TAB_BLURB' => 'Add your content to the section:',
//error messages
'ERROR404' => 'Error 404: Content requested was not found.',
'TAB_RETURN_PROBLEM' => 'There was a problem with your Tab request.',
'DUPLICATE_TABS' => 'Your selected Template has duplicate Tabs of the same name.',
'ILLEGAL_TABS' => 'There are one or more Tabs in your template that use illegal characters. They have been hidden.',
'WARNING' => 'WARNING!',
//helpful messages
'CURRENTLY_EDITING' => 'Clicking a Tab above will SAVE and switch inputs. You are currently editing:',
'CHANGE_TEMPLATE' => 'If you change the Page Template, you will need to SAVE in order for your Tabs to update.',
'TAB_SECTION_NAME' => 'Content for this Tab goes in:',
'PREVIEW_BUTTON' => 'Display Tab Locations.',
// preview page
'SECTION_CODE' => 'This section is populated by the code:',
'REPLACE_CODE' => 'For proper functionality you should change the code to:'
);
break;
}
$SIT_I18N_LOCAL['DEFAULT_TAB'] = bling($SIT_I18N_LOCAL['DEFAULT_TAB']);//	MAke sure the default tab is fancy.
}	/*-------------------------------------------------------------------------------*/?>